<?php

namespace Drupal\library_manager\Form;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\Component\Utility\Unicode;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContextAwarePluginInterface;
use Drupal\Core\Url;

/**
 * Library definition form.
 *
 * @property \Drupal\library_manager\LibraryDefinitionInterface $entity
 */
class LibraryDefinitionForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {

    $form = parent::form($form, $form_state);

    $context_repository = \Drupal::service('context.repository');
    $form_state->setTemporaryValue('gathered_contexts', $context_repository->getAvailableContexts());

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $this->entity->id(),
      '#machine_name' => [
        'exists' => '\Drupal\library_manager\Entity\LibraryDefinition::load',
      ],
      '#disabled' => !$this->entity->isNew(),
    ];

    $form['version'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Version'),
      '#required' => TRUE,
      '#default_value' => $this->entity->get('version'),
    ];

    $datalist = new FormattableMarkup('<datalist id="license"><option value="GNU-GPL-2.0-or-later"><option value="MIT"><option value="Public Domain"></datalist>', []);
    $form['license'] = [
      '#type' => 'textfield',
      '#title' => $this->t('License'),
      '#field_suffix' => $datalist,
      '#attributes' => array('list' => array('license')),
      '#default_value' => $this->entity->get('license'),
    ];

    // JS files.
    $form['js'] = [
      '#type' => 'fieldset',
      '#title' => t('JS files'),
    ];

    $js_header = [
      $this->t('Name'),
      $this->t('Size'),
      $this->t('Type'),
      $this->t('Operations'),
    ];

    $form['js']['table'] = [
      '#type' => 'table',
      '#header' => $js_header,
      '#rows' => [],
      '#empty' => t('JS files are not configured yet.'),
    ];

    $js = $this->entity->get('js');
    foreach ($js as $file_id => $js_data) {

      $route_parameters = [
        'library_definition' => $this->entity->id(),
        'file_id' => $file_id,
      ];

      $operation_links = [
        'edit' => [
          'title' => $this->t('Edit'),
          'url' => Url::fromRoute('entity.library_definition.edit_js_form', $route_parameters),
        ],
        'delete' => [
          'title' => $this->t('Delete'),
          'url' => Url::fromRoute('entity.library_definition.delete_js_form', $route_parameters),
        ],
      ];

      $operation_data = [
        '#type' => 'operations',
        '#links' => $operation_links,
      ];

      $form['js']['table']['#rows'][$file_id] = [
        0 => $js_data['file_name'],
        1 => format_size(Unicode::strlen($js_data['code'])),
        2 => t('Local'),
        'operations' => ['data' => $operation_data],
      ];
    }

    if (!$this->entity->isNew()) {
      $form['js']['add_js'] = [
        '#type' => 'link',
        '#title' => t('Add JS file'),
        '#url' => new Url('entity.library_definition.add_js_form', ['library_definition' => $this->entity->id()]),
        '#attributes' => ['class' => ['button', 'button--small']],
      ];

    }

    // CSS files.
    $form['css'] = [
      '#type' => 'fieldset',
      '#title' => t('CSS files'),
    ];

    $css_header = [
      $this->t('Name'),
      $this->t('Size'),
      $this->t('Type'),
      $this->t('Operations'),
    ];

    $form['css']['table'] = [
      '#type' => 'table',
      '#header' => $css_header,
      '#rows' => [],
      '#empty' => $this->t('CSS files are not configured yet.'),
    ];

    $css = $this->entity->get('css');
    foreach ($css as $file_id => $css_data) {

      $route_parameters = [
        'library_definition' => $this->entity->id(),
        'file_id' => $file_id,
      ];

      $operation_links = [
        'edit' => [
          'title' => $this->t('Edit'),
          'url' => Url::fromRoute('entity.library_definition.edit_css_form', $route_parameters),
        ],
        'delete' => [
          'title' => $this->t('Delete'),
          'url' => Url::fromRoute('entity.library_definition.delete_css_form', $route_parameters),
        ],
      ];

      $operation_data = [
        '#type' => 'operations',
        '#links' => $operation_links,
      ];

      $form['css']['table']['#rows'][$file_id] = [
        0 => $css_data['file_name'],
        1 => format_size(Unicode::strlen($css_data['code'])),
        2 => t('Local'),
        'operations' => ['data' => $operation_data],
      ];
    }

    if (!$this->entity->isNew()) {
      $form['css']['add_css'] = [
        '#type' => 'link',
        '#title' => t('Add CSS file'),
        '#url' => new Url('entity.library_definition.add_css_form', ['library_definition' => $this->entity->id()]),
        '#attributes' => ['class' => ['button', 'button--small']],
      ];

    }

    // Dependencies.
    $form['dependencies'] = [
      '#type' => 'fieldset',
      '#title' => t('Dependencies'),
    ];

    $form['dependencies']['wrapper'] = [
      '#type' => 'container',
      '#id' => 'wrapper',
    ];

    $form['dependencies']['wrapper']['library_dependencies']['#tree'] = TRUE;

    $stored_dependencies = $this->entity->get('library_dependencies');
    $submitted_dependencies = (array) $form_state->getValue('library_dependencies');

    $max_delta = max(count($stored_dependencies), count($submitted_dependencies) + 1);

    for ($delta = 0; $delta < $max_delta; $delta++) {
      $form['dependencies']['wrapper']['library_dependencies'][$delta] = [
        '#type' => 'textfield',
        '#autocomplete_route_name' => 'library_manager.library_autocomplete',
        '#default_value' => isset($stored_dependencies[$delta]) ? $stored_dependencies[$delta] : '',
      ];
    }

    $form['dependencies']['add'] = [
      '#type' => 'button',
      '#name' => 'add_library',
      '#value' => t('Add dependency'),
      '#submit' => [[$this, 'addMoreSubmit']],
      '#attributes' => ['class' => ['button', 'button--small']],
      '#ajax' => [
        'callback' => [$this, 'addMoreCallback'],
        'wrapper' => 'wrapper',
      ],
    ];

    $form['load'] = [
      '#type' => 'checkbox',
      '#title' => t('Load the library automatically according to visibility rules'),
      '#default_value' => $this->entity->get('load'),
    ];

    $form['visibility'] = $this->buildVisibilityInterface([], $form_state);

    return $form;
  }

  /**
   * Dependencies ajax callback.
   */
  public function addMoreCallback(array $form, FormStateInterface $form_state) {
    return $form['dependencies']['wrapper'];
  }

  /**
   * Dependencies submit callback.
   */
  public function addMoreSubmit(array $form, FormStateInterface $form_state) {
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {

    $message = $this->entity->save() == SAVED_NEW ?
      t('Library definition has been added.') : t('Library definition has been updated.');
    drupal_set_message($message);

    $form_state->setRedirectUrl($this->entity->toUrl('collection'));
  }

  /**
   * Helper function for building the visibility UI form.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form array with the visibility UI added in.
   */
  protected function buildVisibilityInterface(array $form, FormStateInterface $form_state) {

    $form['#tree'] = TRUE;

    $form['visibility_tabs'] = [
      '#type' => 'vertical_tabs',
      '#title' => t('Visibility'),
      '#parents' => ['visibility_tabs'],
    ];

    $visibility = $this->entity->getVisibility();

    $condition_manager = \Drupal::service('plugin.manager.condition');

    foreach ($condition_manager->getDefinitions() as $condition_id => $definition) {

      /** @var \Drupal\Core\Condition\ConditionInterface $condition */
      $condition = $condition_manager->createInstance($condition_id, isset($visibility[$condition_id]) ? $visibility[$condition_id] : []);
      $form_state->set(['conditions', $condition_id], $condition);
      $condition_form = $condition->buildConfigurationForm([], $form_state);
      $condition_form['#type'] = 'details';
      $condition_form['#title'] = $condition->getPluginDefinition()['label'];
      $condition_form['#group'] = 'visibility_tabs';
      $form[$condition_id] = $condition_form;
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $this->validateVisibility($form, $form_state);
  }

  /**
   * Helper function to independently validate the visibility UI.
   *
   * @param array $form
   *   A nested array form elements comprising the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  protected function validateVisibility(array $form, FormStateInterface $form_state) {

    // Validate visibility condition settings.
    foreach ($form_state->getValue('visibility') as $condition_id => $values) {
      // All condition plugins use 'negate' as a Boolean in their schema.
      // However, certain form elements may return it as 0/1. Cast here to
      // ensure the data is in the expected type.
      if (array_key_exists('negate', $values)) {
        $values['negate'] = (bool) $values['negate'];
      }

      // Allow the condition to validate the form.
      $condition = $form_state->get(['conditions', $condition_id]);
      $condition_values = (new FormState())
        ->setValues($values);
      $condition->validateConfigurationForm($form, $condition_values);
      // Update the original form values.
      $form_state->setValue(['visibility', $condition_id], $condition_values->getValues());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $entity = $this->entity;

    $dependencies = array_values(array_filter($form_state->getValue('library_dependencies')));

    $entity->set('library_dependencies', $dependencies);

    // Submit visibility condition settings.
    foreach ($form_state->getValue('visibility') as $condition_id => $values) {
      // Allow the condition to submit the form.
      $condition = $form_state->get(['conditions', $condition_id]);
      $condition_values = (new FormState())
        ->setValues($values);
      $condition->submitConfigurationForm($form, $condition_values);

      if ($condition instanceof ContextAwarePluginInterface) {
        $context_mapping = isset($values['context_mapping']) ? $values['context_mapping'] : [];
        $condition->setContextMapping($context_mapping);
      }
      // Update the original form values.
      $condition_configuration = $condition->getConfiguration();
      $form_state->setValue(['visibility', $condition_id], $condition_configuration);
      // Update the visibility conditions on the block.
      $ec = $entity->getVisibilityConditions();
      $entity->getVisibilityConditions()->addInstanceId($condition_id, $condition_configuration);
    }

    // Save the settings of the plugin.
    $entity->save();
  }

}
