<?php

namespace Drupal\library_manager;

use Drupal\Core\Asset\LibraryDiscovery as BaseLibraryDiscovery;
use Drupal\Core\Cache\CacheCollectorInterface;
use Drupal\Core\Extension\ExtensionDiscovery;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Extension\ThemeHandlerInterface;

/**
 * Discovers available asset libraries.
 */
class LibraryDiscovery extends BaseLibraryDiscovery implements LibraryDiscoveryInterface {

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The theme handler service.
   *
   * @var \Drupal\Core\Extension\ThemeHandlerInterface
   */
  protected $themeHandler;

  /**
   * Constructs the controller object.
   *
   * @param \Drupal\Core\Cache\CacheCollectorInterface $library_discovery_collector
   *   The library discovery cache collector.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Extension\ThemeHandlerInterface $theme_handler
   *   The theme handler service.
   */
  public function __construct(CacheCollectorInterface $library_discovery_collector, ModuleHandlerInterface $module_handler, ThemeHandlerInterface $theme_handler) {
    parent::__construct($library_discovery_collector);
    $this->moduleHandler = $module_handler;
    $this->themeHandler = $theme_handler;
  }

  /**
   * {@inheritdoc}
   */
  public function getLibraries() {
    $libraries = [];
    foreach ($this->getEnabledExtensions() as $extension) {
      foreach ($this->getLibrariesByExtension($extension) as $library_name => $library_info) {
        $library_info['extension'] = $extension;
        $library_info['name'] = $library_name;
        $libraries[$library_info['extension'] . '/' . $library_info['name']] = $library_info;
      }
    }

    return $libraries;
  }

  /**
   * Finds enabled extensions.
   *
   * @return array
   *   List of enabled extension including core.
   */
  protected function getEnabledExtensions() {

    $enabled_extensions = ['core'];

    $listing = new ExtensionDiscovery(DRUPAL_ROOT);
    $listing->setProfileDirectories([]);

    $extensions = $listing->scan('module', TRUE);
    $extensions += $listing->scan('profile', TRUE);
    $extensions += $listing->scan('theme', TRUE);

    foreach ($extensions as $extension) {
      $extension_name = $extension->getName();

      // Do not check libraries for disabled extensions.
      if ($extension->getType() == 'theme') {
        if (!$this->themeHandler->themeExists($extension_name)) {
          continue;
        }
      }
      elseif (!$this->moduleHandler->moduleExists($extension_name)) {
        continue;
      }

      $enabled_extensions[] = $extension_name;
    }

    return $enabled_extensions;
  }

}
